library ieee;
use ieee.std_logic_1164.all;
use IEEE.numeric_std.all;


entity ADC_VGA_0_3 is
port (CLK: in std_logic;
		ADC_in: in std_logic_vector(7 downto 0);
		JC: in std_logic_vector(7 downto 0);
		Hsync, Vsync, ADCCLK: out std_logic;
		vgaRed, vgaGreen:  out std_logic_vector(2 downto 0);
		vgaBlue:  out std_logic_vector(1 downto 0);
		khz_100: out std_logic;
		mhz_1: out std_logic;
		sw:   in std_logic_vector(7 downto 4)); --7
end  ADC_VGA_0_3;

architecture behav of  ADC_VGA_0_3 is

COMPONENT dcm75
	PORT(
		CLKIN_IN : IN std_logic;          
		CLKFX_OUT : OUT std_logic;
		CLKIN_IBUFG_OUT : OUT std_logic;
		CLK0_OUT : OUT std_logic;
		CLK2X_OUT : OUT std_logic
		);
END COMPONENT;

component vga is
  port(clk75_in, bin  : in std_logic;
  		 horizontal_counter : out std_logic_vector (10 downto 0);
		 vertical_counter   : out std_logic_vector (9 downto 0);
       red_out   : out std_logic_vector(2 downto 0);
       green_out : out std_logic_vector(2 downto 0);
       blue_out  : out std_logic_vector(1 downto 0);
       hs_out    : out std_logic;
       vs_out    : out std_logic);
end component;

component ram_32k IS
  PORT (
    clka : IN STD_LOGIC;
    wea : IN STD_LOGIC_VECTOR(0 DOWNTO 0);
    addra : IN STD_LOGIC_VECTOR(14 DOWNTO 0);
    dina : IN STD_LOGIC_VECTOR(7 DOWNTO 0);
    douta : OUT STD_LOGIC_VECTOR(7 DOWNTO 0);
    clkb : IN STD_LOGIC;
    web : IN STD_LOGIC_VECTOR(0 DOWNTO 0);
    addrb : IN STD_LOGIC_VECTOR(14 DOWNTO 0);
    dinb : IN STD_LOGIC_VECTOR(7 DOWNTO 0);
    doutb : OUT STD_LOGIC_VECTOR(7 DOWNTO 0)
  );
END component;

component Debenc is
    Port ( clk      : in  STD_LOGIC;
           encin    : in  STD_LOGIC_VECTOR(1 downto 0);
           r	     : out  STD_LOGIC;
			  l	     : out  STD_LOGIC);
end component;

signal hcv: std_logic_vector (10 downto 0);
signal vcv: std_logic_vector (9 downto 0);
signal hci_CLK75: integer range 0 to 2047;
signal vci_CLK75: integer range 0 to 1023;

signal rai_32k, wai_32k: integer range 0 to 32767;
signal wa_32k, ra_32k: std_logic_vector (14 downto 0);
signal di_32k, do_32k: std_logic_vector (7 downto 0);
signal we_32k: std_logic_vector (0 downto 0);

signal rai_1k, wai_1k: integer range 0 to 1023;
signal rai_1k_all, wai_1k_all: integer range 0 to 1023;

signal buff00, buff01, buff02, buff03, buff04, buff05, buff06, buff07,
		 buff08, buff09, buff0a ,buff0b, buff0c, buff0d, buff0e, buff0f,
		 buff10, buff11, buff12, buff13, buff14, buff15, buff16, buff17,
		 buff18, buff19, buff1a ,buff1b, buff1c, buff1d, buff1e, buff1f,
		 buff20, buff21, buff22, buff23, buff24, buff25, buff26, buff27,
		 buff28, buff29, buff2a ,buff2b, buff2c, buff2d, buff2e, buff2f,
		 buff30, buff31, buff32, buff33, buff34, buff35, buff36, buff37,
		 buff38, buff39, buff3a ,buff3b, buff3c, buff3d, buff3e, buff3f: integer range 0 to 255;
		 
signal sum000, sum001, sum002, sum003, sum004, sum005, sum006, sum007: integer range 0 to 511;
signal sum100, sum101, sum102, sum103, sum104, sum105, sum106, sum107: integer range 0 to 511;
signal sum200, sum201, sum202, sum203, sum204, sum205, sum206, sum207: integer range 0 to 511;
signal sum300, sum301, sum302, sum303, sum304, sum305, sum306, sum307: integer range 0 to 511;

signal sum010, sum011, sum012, sum013: integer range 0 to 1023;
signal sum110, sum111, sum112, sum113: integer range 0 to 1023;
signal sum210, sum211, sum212, sum213: integer range 0 to 1023;
signal sum310, sum311, sum312, sum313: integer range 0 to 1023;

signal sum020, sum021: integer range 0 to 2047;
signal sum120, sum121: integer range 0 to 2047;
signal sum220, sum221: integer range 0 to 2047;
signal sum320, sum321: integer range 0 to 2047;

signal sum0: integer range 0 to 4095;
signal sum1: integer range 0 to 4095;
signal sum2: integer range 0 to 4095;
signal sum3: integer range 0 to 4095;

signal summe01: integer range 0 to 8191;
signal summe23: integer range 0 to 8191;

signal summe0123: integer range 0 to 16383;

signal sum000_buff: integer range 0 to 255;
signal sum010_buff: integer range 0 to 255;
signal sum020_buff: integer range 0 to 255;
signal sum0_buff  : integer range 0 to 255;
signal sum01_buff : integer range 0 to 255;
signal sum0123_buff: integer range 0 to 255;

signal point_new, point_old: integer range 0 to 511;
signal point_new_all, point_old_all: integer range 0 to 511;

signal ADCC_v: std_logic_vector (5 downto 0);
signal ADCC: integer range 0 to 63;
signal swi_ADCCLK: integer range 0 to 15;
signal swi_CLK75: integer range 0 to 15;
signal swi: integer range 0 to 15;

signal scale_sum: integer range 0 to 8191;
signal scale_sum_v: std_logic_vector (12 downto 0);

signal scale_sum_all: integer range 0 to 8191;
signal scale_sum_v_all: std_logic_vector (12 downto 0);

signal point,bool,bool0,bool1,CLK100,CLK75,CLK50,edge,sign,rise_store: std_logic;

signal trigger_y: integer range 1 to 30 := 16;
signal trigger_x: integer range 0 to 1023 := 1;
signal bar_y, bar_x0, bar_x1: std_logic;

signal one_shot, store, rise_sw4: std_logic;
signal frame_counter: integer range 0 to 2;

signal khz_100_counter: integer range 0 to 499;
signal mhz_1_counter: integer range 0 to 49;

type ram_1024 is array (1023 downto 0) of std_logic_vector(7 downto 0);
signal ram_1k : ram_1024;
signal ram_1k_all : ram_1024;

signal en0_fall0, en0_fall1, en1_fall0, en1_fall1, en2_fall0, en2_fall1, en3_fall0, en3_fall1 : std_logic := '0';

signal en1_r, en1_l, en2_r, en2_l, en3_r, en3_l: std_logic;

begin

debenc1: Debenc PORT MAP(CLK75, JC(3 downto 2), en1_r, en1_l);
debenc2: Debenc PORT MAP(CLK75, JC(5 downto 4), en2_r, en2_l);
debenc3: Debenc PORT MAP(CLK75, JC(7 downto 6), en3_r, en3_l);

	
Inst_dcm75: dcm75 PORT MAP(
		CLKIN_IN => CLK,
		CLKFX_OUT => CLK75,
		CLKIN_IBUFG_OUT => CLK50,
		CLK0_OUT => open,
		CLK2X_OUT => CLK100
		);
		
v0: vga port map (CLK75, point, hcv, vcv, vgaRed, vgaGreen, vgaBlue, Hsync, Vsync);

r0: ram_32k PORT MAP(
    clka => CLK100,
    wea => we_32k,
    addra => wa_32k,
    dina => di_32k,
    douta => open,
    clkb => CLK75,
    web => "0",
    addrb => ra_32k,
    dinb => "00000000",
    doutb => do_32k
    );
	 
ADCCLK <= CLK100;
point <= bool or bar_y or bar_x0 or bar_x1 or bool0 or bool1;

mhz_1 <= '0' when mhz_1_counter < 25 else '1';
khz_100 <= '0' when khz_100_counter < 250 else '1';

store <= '1' when (sw(7) = '1' or sw(4) = '1') else '0';

process
	variable sr : std_logic_vector (3 downto 0) := "0000";
	variable sr_sw4 : std_logic_vector (3 downto 0) := "0000";
	begin
    wait until rising_edge(CLK100);
	 
	swi_ADCCLK <= swi; --to_integer(unsigned(sw(3 downto 0)));
	ADCC_v <= std_logic_vector(to_unsigned(ADCC,6));
	wa_32k <= std_logic_vector(to_unsigned(wai_32k,15));

--	hci_ADCCLK <= to_integer(unsigned(hcv));
--	vci_ADCCLK <= to_integer(unsigned(vcv));
	 
	if ADCC < 63 then
		ADCC <= ADCC +1;
	else
		ADCC <= 0;
	end if;
	
	if wai_32k < 32767 then
		if sw(7) = '0' then
			if 	swi_ADCCLK <= 5 					then
				di_32k <= std_logic_vector(to_unsigned(buff00,8)); --00
				wai_32k <= wai_32k +1;
			elsif swi_ADCCLK = 6 and ADCC_v(0) = '1' then
				ADCC <= 0; di_32k <= std_logic_vector(to_unsigned(sum000_buff,8));
				wai_32k <= wai_32k +1;
			elsif swi_ADCCLK = 7 and ADCC_v(1 downto 0) = "11" then
				ADCC <= 0; di_32k <= std_logic_vector(to_unsigned(sum010_buff,8));
				wai_32k <= wai_32k +1;
			elsif swi_ADCCLK = 8 and ADCC_v(2 downto 0) = "111" then
				ADCC <= 0; di_32k <= std_logic_vector(to_unsigned(sum020_buff,8));
				wai_32k <= wai_32k +1;
			elsif swi_ADCCLK = 9 and ADCC_v(3 downto 0) = "1111" then
				ADCC <= 0; di_32k <= std_logic_vector(to_unsigned(sum0_buff,8));
				wai_32k <= wai_32k +1;
			elsif swi_ADCCLK = 10 and ADCC_v(4 downto 0) = "11111" then
				ADCC <= 0; di_32k <= std_logic_vector(to_unsigned(sum01_buff,8));
				wai_32k <= wai_32k +1;
			elsif swi_ADCCLK > 10 and ADCC_v(5 downto 0) = "111111" then
				ADCC <= 0; di_32k <= std_logic_vector(to_unsigned(sum0123_buff,8));
				wai_32k <= wai_32k +1;
			end if;
			we_32k <= "1";
		elsif sw(7) = '1' then
			we_32k <= "0";
		end if;
	elsif wai_32k = 32767 and (frame_counter = 0) and ((sw(4) = '1' and one_shot = '1') or sw(4) = '0') then -- and vci_ADCCLK > 35 and vci_ADCCLK < 200 then
		if sw(6) = '0' then
			if edge = '1' then
				wai_32k <= 0;
				one_shot <= '0';
			end if;
		elsif sw(6) = '1' then
			wai_32k <= 0;
			one_shot <= '0';
		end if;
	end if;

	buff00 <= to_integer(unsigned(ADC_in));
	
	buff01 <= buff00; buff02 <= buff01; buff03 <= buff02; buff04 <= buff03; buff05 <= buff04; buff06 <= buff05; buff07 <= buff06;
	buff08 <= buff07; buff09 <= buff08; buff0a <= buff09; buff0b <= buff0a; buff0c <= buff0b; buff0d <= buff0c; buff0e <= buff0d;
	buff0f <= buff0e; buff10 <= buff0f; buff11 <= buff10; buff12 <= buff11; buff13 <= buff12; buff14 <= buff13; buff15 <= buff14;
	buff16 <= buff15; buff17 <= buff16; buff18 <= buff17; buff19 <= buff18; buff1a <= buff19; buff1b <= buff1a; buff1c <= buff1b;
	buff1d <= buff1c; buff1e <= buff1d; buff1f <= buff1e; buff20 <= buff1f; buff21 <= buff20; buff22 <= buff21; buff23 <= buff22;
	buff24 <= buff23; buff25 <= buff24; buff26 <= buff25; buff27 <= buff26; buff28 <= buff27; buff29 <= buff28; buff2a <= buff29;
	buff2b <= buff2a; buff2c <= buff2b; buff2d <= buff2c; buff2e <= buff2d; buff2f <= buff2e;	buff30 <= buff2f; buff31 <= buff30;
	buff32 <= buff31; buff33 <= buff32; buff34 <= buff33; buff35 <= buff34; buff36 <= buff35; buff37 <= buff36; buff38 <= buff37;
	buff39 <= buff38; buff3a <= buff39; buff3b <= buff3a; buff3c <= buff3b; buff3d <= buff3c; buff3e <= buff3d; buff3f <= buff3e;			
	
	sum000 <= buff00 + buff01; sum001 <= buff02 + buff03; sum002 <= buff04 + buff05; sum003 <= buff06 + buff07;
	sum004 <= buff08 + buff09; sum005 <= buff0a + buff0b; sum006 <= buff0c + buff0d; sum007 <= buff0e + buff0f;
	sum100 <= buff10 + buff11; sum101 <= buff12 + buff13; sum102 <= buff14 + buff15; sum103 <= buff16 + buff17;
	sum104 <= buff18 + buff19; sum105 <= buff1a + buff1b; sum106 <= buff1c + buff1d; sum107 <= buff1e + buff1f;
	sum200 <= buff20 + buff21; sum201 <= buff22 + buff23; sum202 <= buff24 + buff25; sum203 <= buff26 + buff27;
	sum204 <= buff28 + buff29; sum205 <= buff2a + buff2b; sum206 <= buff2c + buff2d; sum207 <= buff2e + buff2f;
	sum300 <= buff30 + buff31; sum301 <= buff32 + buff33; sum302 <= buff34 + buff35; sum303 <= buff36 + buff37;
	sum304 <= buff38 + buff39; sum305 <= buff3a + buff3b; sum306 <= buff3c + buff3d; sum307 <= buff3e + buff3f;
	
	sum010 <= sum000 + sum001; sum011 <= sum002 + sum003; sum012 <= sum004 + sum005; sum013 <= sum006 + sum007;
	sum110 <= sum100 + sum101; sum111 <= sum102 + sum103; sum112 <= sum104 + sum105; sum113 <= sum106 + sum107;
	sum210 <= sum200 + sum201; sum211 <= sum202 + sum203; sum212 <= sum204 + sum205; sum213 <= sum206 + sum207;
	sum310 <= sum300 + sum301; sum311 <= sum302 + sum303; sum312 <= sum304 + sum305; sum313 <= sum306 + sum307;

	sum020 <= sum010 + sum011; sum021 <= sum012 + sum013;
	sum120 <= sum110 + sum111; sum121 <= sum112 + sum113;
	sum220 <= sum210 + sum211; sum221 <= sum212 + sum213;
	sum320 <= sum310 + sum311; sum321 <= sum312 + sum313;
	
	sum0 <= sum020 + sum021;
	sum1 <= sum120 + sum121;
	sum2 <= sum220 + sum221;
	sum3 <= sum320 + sum321;
	
	summe01 <= sum0 + sum1;
	summe23 <= sum2 + sum3;
	
	summe0123 <= summe01 + summe23;
	
	sum000_buff  <= sum000 /2; --000
	sum010_buff  <= sum010 /4; --010
	sum020_buff  <= sum020 /8; --020
	sum0_buff    <= sum0 /16;  --0
	sum01_buff   <= summe01 /32;  --0
	sum0123_buff <= summe0123 /64;  --0
	
	if sw(5) = '1' then
		if summe01 < summe23 then
			sign <= '0';
		else
			sign <= '1';
		end if;
	elsif sw(5) = '0' then
		if	summe01 < summe23 then
			if (swi_ADCCLK <=  5 and  	    buff00 > (trigger_y *8)-8 and 		 buff00 < (trigger_y *8)+8) or
				(swi_ADCCLK  =  6 and  sum000_buff > (trigger_y *8)-8 and  sum000_buff < (trigger_y *8)+8) or
				(swi_ADCCLK  =  7 and  sum010_buff > (trigger_y *8)-8 and  sum010_buff < (trigger_y *8)+8) or
				(swi_ADCCLK  =  8 and  sum020_buff > (trigger_y *8)-8 and  sum020_buff < (trigger_y *8)+8) or
				(swi_ADCCLK  =  9 and    sum0_buff > (trigger_y *8)-8 and 	 sum0_buff < (trigger_y *8)+8) or
				(swi_ADCCLK  = 10 and   sum01_buff > (trigger_y *8)-8 and 	sum01_buff < (trigger_y *8)+8) or
				(swi_ADCCLK  > 10 and sum0123_buff > (trigger_y *8)-8 and sum0123_buff < (trigger_y *8)+8) then
				sign <= '1';
			end if;
		elsif summe01 > summe23 then
			if (swi_ADCCLK <=  5 and 	    buff00 > (trigger_y *8)-8 and 		 buff00 < (trigger_y *8)+8) or
				(swi_ADCCLK  =  6 and  sum000_buff > (trigger_y *8)-8 and  sum000_buff < (trigger_y *8)+8) or
				(swi_ADCCLK  =  7 and  sum010_buff > (trigger_y *8)-8 and  sum010_buff < (trigger_y *8)+8) or
				(swi_ADCCLK  =  8 and  sum020_buff > (trigger_y *8)-8 and  sum020_buff < (trigger_y *8)+8) or
				(swi_ADCCLK  =  9 and    sum0_buff > (trigger_y *8)-8 and 	 sum0_buff < (trigger_y *8)+8) or
				(swi_ADCCLK  = 10 and   sum01_buff > (trigger_y *8)-8 and   sum01_buff < (trigger_y *8)+8) or
				(swi_ADCCLK  > 10 and sum0123_buff > (trigger_y *8)-8 and sum0123_buff < (trigger_y *8)+8) then
				sign <= '0';
			end if;
		end if;
	end if;

	if rise_sw4 = '1' then
		one_shot <= '1';
	end if;
	
	edge <= not sr(2) and sr(3); --fall
	sr := sr(2 downto 0) & sign;
	
	rise_sw4 <= not sr_sw4(3) and sr_sw4(2); --rise
	sr_sw4 := sr_sw4(2 downto 0) & sw(4);
			
end process;


process
variable en0_sr0, en0_sr1, en1_sr0, en1_sr1, en2_sr0, en2_sr1, en3_sr0, en3_sr1, sr_store : std_logic_vector (3 downto 0) := "0000";

	begin
   wait until rising_edge(CLK75);
	

	if hci_CLK75 = 1 and vci_CLK75 = 36 then
		if frame_counter < 2 then
			frame_counter <= frame_counter +1;
		elsif frame_counter = 2 then
			frame_counter <= 0;
		end if;
	end if;
		
	 hci_CLK75 <= to_integer(unsigned(hcv));
	 vci_CLK75 <= to_integer(unsigned(vcv));
	 ra_32k <= std_logic_vector(to_unsigned(rai_32k,15));
	 
-- 32k BRAM lesen und in 1k BRAM schreiben -------------------------------------------------------------------------------------
	
	if (swi_CLK75 = 0 								        ) or
		(swi_CLK75 = 1 and ra_32k(0) 		     =     '1') or
		(swi_CLK75 = 2 and ra_32k(1 downto 0) =    "11") or
		(swi_CLK75 = 3 and ra_32k(2 downto 0) =   "111") or
		(swi_CLK75 = 4 and ra_32k(3 downto 0) =  "1111") or
		(swi_CLK75 >= 5 and ra_32k(4 downto 0) = "11111") then
			scale_sum 	<= to_integer(unsigned(do_32k));
			scale_sum_v <= std_logic_vector(to_unsigned(scale_sum,13));
			if wai_1k < 1023 then
				wai_1k <= wai_1k +1;
			end if;
	else
		scale_sum <= scale_sum + to_integer(unsigned(do_32k));
	end if;
	
	
	if ra_32k(4 downto 0) = "11111" then
			scale_sum_all 	<= to_integer(unsigned(do_32k));
			scale_sum_v_all <= std_logic_vector(to_unsigned(scale_sum_all,13));
			if wai_1k_all < 1023 then
				wai_1k_all <= wai_1k_all +1;
			end if;
	else
		scale_sum_all <= scale_sum_all + to_integer(unsigned(do_32k));
	end if;
	
	ram_1k_all(wai_1k_all) <= scale_sum_v_all(12 downto 5);

	if wai_1k <  1023 then
		if 	swi_CLK75 =  0 then ram_1k(wai_1k) <= scale_sum_v( 7 downto 0);-- wai_1k <= rai_32k;
		elsif swi_CLK75 =  1 then ram_1k(wai_1k) <= scale_sum_v( 8 downto 1);-- wai_1k <= rai_32k /2;
		elsif swi_CLK75 =  2 then ram_1k(wai_1k) <= scale_sum_v( 9 downto 2);-- wai_1k <= rai_32k /4;
		elsif swi_CLK75 =  3 then ram_1k(wai_1k) <= scale_sum_v(10 downto 3);-- wai_1k <= rai_32k /8;
		elsif swi_CLK75 =  4 then ram_1k(wai_1k) <= scale_sum_v(11 downto 4);-- wai_1k <= rai_32k /16;
		elsif swi_CLK75 >= 5 then ram_1k(wai_1k) <= scale_sum_v(12 downto 5); end if;-- wai_1k <= rai_32k /32; end if;
	end if;
	
	if rai_32k < 32767 then
		rai_32k <= rai_32k +1;
	end if;
	
	if rise_store = '1' then
		wai_1k_all <= 0;
		rai_32k <= 0;
	end if;

	if vci_CLK75 = 1 and frame_counter = 2 and rai_32k = 32767 then --and hci_CLK75 = 1
		swi_CLK75 <= swi; --to_integer(unsigned(sw(3 downto 0)));
		wai_1k <= 0;
		
		if swi_CLK75 =  0 then rai_32k <= trigger_x*16 + trigger_x*8 + trigger_x*4 + trigger_x*2 + trigger_x; end if;
		if swi_CLK75 =  1 then rai_32k <= trigger_x*16 + trigger_x*8 + trigger_x*4 + trigger_x*2; end if;
		if swi_CLK75 =  2 then rai_32k <= trigger_x*16 + trigger_x*8 + trigger_x*4; end if;
		if swi_CLK75 =  3 then rai_32k <= trigger_x*16 + trigger_x*8; end if;
		if swi_CLK75 =  4 then rai_32k <= trigger_x*16; end if;
		if swi_CLK75 >= 5 then rai_32k <= 0; end if;
		
	end if;

-- 1k BRAM lesen und bool ausgeben ---------------------------------------------------------------------------------------------

	if  	 (hci_CLK75 >= 270)--279
		and (hci_CLK75 < 1294)--1304
		and (vci_CLK75 > 35) 					then
		
		rai_1k <= hci_CLK75 - 270; --280
		rai_1k_all <= hci_CLK75 - 270; --280

		point_new <= to_integer(unsigned(ram_1k(rai_1k))) *2;
		point_old <= point_new;
		
		point_new_all <= to_integer(unsigned(ram_1k_all(rai_1k_all))) /2;
		point_old_all <= point_new_all;

		if point_new < point_old then
			if 580 - vci_CLK75 < 	  point_old and 580 - vci_CLK75 >= 	   point_new then
					bool0 <= '1';
			else bool0 <= '0';
			end if;
		elsif point_new > point_old then
			if 580 - vci_CLK75 > 	  point_old and 580 - vci_CLK75 <= 	   point_new then
					bool0 <= '1';
			else bool0 <= '0';
			end if;
		elsif point_new = point_old then
			if 580 - vci_CLK75 = point_new then
					bool0 <= '1';
			else bool0 <= '0';
			end if;
		end if;

		if point_new_all < point_old_all then
			if 750 - vci_CLK75 < point_old_all and 750 - vci_CLK75 >= point_new_all and store = '1' then
					bool1 <= '1';
			else bool1 <= '0';
			end if;
		elsif point_new_all > point_old_all then
			if 750 - vci_CLK75 > point_old_all and 750 - vci_CLK75 <= point_new_all and store = '1' then
					bool1 <= '1';
			else bool1 <= '0';
			end if;
		elsif point_new_all = point_old_all then
			if 750 - vci_CLK75 = point_new_all and store = '1' then
					bool1 <= '1';
			else bool1 <= '0';
			end if;
		end if;
		
--		if hcv(0) = '0' and (vci_CLK75 = 585 or vci_CLK75 = 63) then
--			bool <= '1';
--		else
--			bool <= '0';
--		end if;
		
		if hcv(1 downto 0) = "00" and (vci_CLK75 = 755 or vci_CLK75 = 617) and store = '1' then
			bool <= '1';
		else
			bool <= '0';
		end if;
	else
		bool <= '0';
		bool0 <= '0';
		bool1 <= '0';
	end if;
	
-- trigger_y -------------------------------------------------------------------------------------------------------------------
	
	if en1_r = '1' and trigger_y < 30 then trigger_y <= trigger_y +1; end if;
	if en1_l = '1' and trigger_y  > 1 then trigger_y <= trigger_y -1; end if;
	
	if hci_CLK75 < 280 and store = '0' then
		if 	((580 - vci_CLK75) > (trigger_y *16)+16 and (580 - vci_CLK75) < (trigger_y *16)+32) or
				((580 - vci_CLK75) < (trigger_y *16)-16 and (580 - vci_CLK75) > (trigger_y *16)-32) then
			bar_y <= '1';
		else
			bar_y <= '0';
		end if;
	else
			bar_y <= '0';
	end if;

-- trigger_x -------------------------------------------------------------------------------------------------------------------

	if en3_r = '1' then --en3_fall1 = '1' and JC(6) = '1' then --deb_btn(0) = '1' then
		if swi_CLK75 =  0 and trigger_x < 1022 then trigger_x <= trigger_x +  2; end if;
		if swi_CLK75 =  1 and trigger_x < 1020 then trigger_x <= trigger_x +  4; end if;
		if swi_CLK75 =  2 and trigger_x < 1016 then trigger_x <= trigger_x +  8; end if;
		if swi_CLK75 =  3 and trigger_x < 1008 then trigger_x <= trigger_x + 16; end if;
		if swi_CLK75 >=  4 and trigger_x <  992 then trigger_x <= trigger_x + 32; end if;
	end if;

	if en3_l = '1' then --en3_fall0 = '1' and JC(7) = '1' then --deb_btn(1) = '1' then
		if swi_CLK75 =  0 and trigger_x >  2 then trigger_x <= trigger_x -  2; end if;
		if swi_CLK75 =  1 and trigger_x >  4 then trigger_x <= trigger_x -  4; end if;
		if swi_CLK75 =  2 and trigger_x >  7 then trigger_x <= trigger_x -  8; end if;
		if swi_CLK75 =  3 and trigger_x > 15 then trigger_x <= trigger_x - 16; end if;
		if swi_CLK75 >=  4 and trigger_x > 31 then trigger_x <= trigger_x - 32; end if;
	end if;
	
	if en3_r = '0' and en3_l = '0' then --en3_fall0 = '0' and en3_fall1 = '0' then --deb_btn(1 downto 0) = "00" then
		if swi_CLK75 =  0 and trigger_x <  2 then trigger_x <= 0; end if;
		if swi_CLK75 =  1 and trigger_x <  4 then trigger_x <= 0; end if;
		if swi_CLK75 =  2 and trigger_x <  8 then trigger_x <= 0; end if;
		if swi_CLK75 =  3 and trigger_x < 16 then trigger_x <= 0; end if;
		if swi_CLK75 >=  4 and trigger_x < 32 then trigger_x <= 0; end if;

		if swi_CLK75 =  0 and trigger_x > 1021 then trigger_x <= 1023; end if;
		if swi_CLK75 =  1 and trigger_x > 1019 then trigger_x <= 1023; end if;
		if swi_CLK75 =  2 and trigger_x > 1015 then trigger_x <= 1023; end if;
		if swi_CLK75 =  3 and trigger_x > 1007 then trigger_x <= 1023; end if;
		if swi_CLK75 >=  4 and trigger_x >  991 then trigger_x <= 1023; end if;
	end if;
	
	if vci_CLK75 < 756 and vci_CLK75 > 616 and store = '1' then
		if (swi_CLK75 =   0 and (hci_CLK75 = 270 + trigger_x/2 + trigger_x/4 + trigger_x/8 + trigger_x/16 + trigger_x/32 or hci_CLK75 = 302 + trigger_x/2 + trigger_x/4 + trigger_x/8 + trigger_x/16 + trigger_x/32)) or --step: 62
			(swi_CLK75 =   1 and (hci_CLK75 = 270 + trigger_x/2 + trigger_x/4 + trigger_x/8 + trigger_x/16 					  or hci_CLK75 = 334 + trigger_x/2 + trigger_x/4 + trigger_x/8 + trigger_x/16)) or --step: 60
			(swi_CLK75 =   2 and (hci_CLK75 = 270 + trigger_x/2 + trigger_x/4 + trigger_x/8 										  or hci_CLK75 = 398 + trigger_x/2 + trigger_x/4 + trigger_x/8)) or --step: 56
			(swi_CLK75 =   3 and (hci_CLK75 = 270 + trigger_x/2 + trigger_x/4 														  or hci_CLK75 = 526 + trigger_x/2 + trigger_x/4)) or
			(swi_CLK75 >=  4 and (hci_CLK75 = 270 + trigger_x/2																			  or hci_CLK75 = 782 + trigger_x/2)) then
			bar_x0 <= '1';
		else
			bar_x0 <= '0';
		end if;
	else
			bar_x0 <= '0';
	end if;
		
	if (vci_CLK75 = 755 or vci_CLK75 = 617) and store = '1' then
		if (swi_CLK75 =   0 and hci_CLK75 > 270 + trigger_x/2 + trigger_x/4 + trigger_x/8 + trigger_x/16 + trigger_x/32 and hci_CLK75 < 302 + trigger_x/2 + trigger_x/4 + trigger_x/8 + trigger_x/16 + trigger_x/32) or --step: 62
			(swi_CLK75 =   1 and hci_CLK75 > 270 + trigger_x/2 + trigger_x/4 + trigger_x/8 + trigger_x/16 					 and hci_CLK75 < 334 + trigger_x/2 + trigger_x/4 + trigger_x/8 + trigger_x/16) or --step: 60
			(swi_CLK75 =   2 and hci_CLK75 > 270 + trigger_x/2 + trigger_x/4 + trigger_x/8 										 and hci_CLK75 < 398 + trigger_x/2 + trigger_x/4 + trigger_x/8) or --step: 56
			(swi_CLK75 =   3 and hci_CLK75 > 270 + trigger_x/2 + trigger_x/4 														  	 and hci_CLK75 < 526 + trigger_x/2 + trigger_x/4) or
			(swi_CLK75 >=  4 and hci_CLK75 > 270 + trigger_x/2																			    and hci_CLK75 < 782 + trigger_x/2) then
			bar_x1 <= '1';
		else
			bar_x1 <= '0';
		end if;
	else
			bar_x1 <= '0';
	end if;
	
-- swi/zoom_encoder ------------------------------------------------------------------------------------------------------------
	
	if en2_r = '1' and swi < 15 then swi <= swi +1; end if;
	if en2_l = '1' and swi >  0 then swi <= swi -1; end if;

	rise_store <= not sr_store(3) and sr_store(2); --rise
	sr_store := sr_store(2 downto 0) & store;

end process;


process
	begin
   wait until rising_edge(CLK50);
	
	if khz_100_counter < 499 then
		khz_100_counter <= khz_100_counter +1;
	else
		khz_100_counter <= 0;
	end if;
	 
	if mhz_1_counter < 49 then
		mhz_1_counter <= mhz_1_counter +1;
	else
		mhz_1_counter <= 0;
	end if;
	 
end process;

end behav;
			